@@ -196,11 +196,11 @@ append_gexiv2_tag (NautilusImagesPropertiesPage  *page,
         {
             g_autofree char *tag_value = NULL;
 
-            tag_value = gexiv2_metadata_get_tag_interpreted_string (page->md, *i);
+            tag_value = gexiv2_metadata_try_get_tag_interpreted_string (page->md, *i, NULL);
 
             if (description == NULL)
             {
-                description = gexiv2_metadata_get_tag_description (*i);
+                description = gexiv2_metadata_try_get_tag_description (*i, NULL);
             }
 
             /* don't add empty tags - try next one */
@@ -262,7 +262,7 @@ append_gexiv2_info (NautilusImagesPropertiesPage *page)
     append_gexiv2_tag (page, rights, _("Copyright"));
     append_gexiv2_tag (page, rating, _("Rating"));
 
-    if (gexiv2_metadata_get_gps_info (page->md, &longitude, &latitude, &altitude))
+    if (gexiv2_metadata_try_get_gps_info (page->md, &longitude, &latitude, &altitude, NULL))
     {
         g_autofree char *gps_coords = NULL;
 
