@@ -399,7 +399,12 @@ nautilus_floating_bar_constructed (GObject *obj)
     w = gtk_spinner_new ();
     gtk_box_pack_start (GTK_BOX (box), w, FALSE, FALSE, 0);
     gtk_widget_set_visible (w, self->show_spinner);
-    gtk_spinner_start (GTK_SPINNER (w));
+    /* As a workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/1025,
+     * ensure the spinner animates if and only if it's visible, to reduce CPU
+     * usage. */
+    g_object_bind_property (obj, "show-spinner",
+                            w, "active",
+                            G_BINDING_SYNC_CREATE);
     self->spinner = w;
 
     gtk_widget_set_size_request (w, 16, 16);
