@@ -54,12 +54,6 @@ enum
     NAUTILUS_DND_NTARGETS
 };
 
-static const GtkTargetEntry drag_types [] =
-{
-    { NAUTILUS_DND_URI_LIST_TYPE, 0, NAUTILUS_DND_URI_LIST },
-    { NAUTILUS_DND_TEXT_PLAIN_TYPE, 0, NAUTILUS_DND_TEXT_PLAIN },
-};
-
 static const GtkTargetEntry drop_types [] =
 {
     { NAUTILUS_DND_URI_LIST_TYPE, 0, NAUTILUS_DND_URI_LIST },
@@ -72,6 +66,7 @@ typedef struct _NautilusLocationEntryPrivate
     GFilenameCompleter *completer;
 
     guint idle_id;
+    gboolean idle_insert_completion;
 
     GFile *last_location;
 
@@ -79,6 +74,10 @@ typedef struct _NautilusLocationEntryPrivate
     gboolean setting_special_text;
     gchar *special_text;
     NautilusLocationEntryAction secondary_action;
+
+    GtkEntryCompletion *completion;
+    GtkListStore *completions_store;
+    GtkCellRenderer *completion_cell;
 } NautilusLocationEntryPrivate;
 
 enum
@@ -220,6 +219,9 @@ nautilus_location_entry_set_location (NautilusLocationEntry *entry,
 
     nautilus_location_entry_update_action (entry);
 
+    /* invalidate the completions list */
+    gtk_list_store_clear (priv->completions_store);
+
     g_free (uri);
     g_free (formatted_uri);
 }
@@ -346,62 +348,125 @@ drag_data_get_callback (GtkWidget        *widget,
         break;
 
         default:
+        {
             g_assert_not_reached ();
+        }
     }
     g_free (uri);
     g_object_unref (location);
 }
 
-/* routine that performs the tab expansion.  Extract the directory name and
- *  incomplete basename, then iterate through the directory trying to complete it.  If we
- *  find something, add it to the entry */
 
+static void
+set_prefix_dimming (GtkCellRenderer *completion_cell,
+                    char            *user_location)
+{
+    g_autofree char *location_basename = NULL;
+    PangoAttrList *attrs;
+    PangoAttribute *attr;
+
+    /* Dim the prefixes of the completion rows, leaving the basenames
+     * highlighted. This makes it easier to find what you're looking for.
+     *
+     * Perhaps a better solution would be to *only* show the basenames, but
+     * it would take a reimplementation of GtkEntryCompletion to align the
+     * popover. */
+
+    location_basename = g_path_get_basename (user_location);
+
+    attrs = pango_attr_list_new ();
+
+    /* 55% opacity. This is the same as the dim-label style class in Adwaita. */
+    attr = pango_attr_foreground_alpha_new (36045);
+    attr->end_index = strlen (user_location) - strlen (location_basename);
+    pango_attr_list_insert (attrs, attr);
+
+    g_object_set (completion_cell, "attributes", attrs, NULL);
+    pango_attr_list_unref (attrs);
+}
+
+
+/* Update the path completions list based on the current text of the entry. */
 static gboolean
-try_to_expand_path (gpointer callback_data)
+update_completions_store (gpointer callback_data)
 {
     NautilusLocationEntry *entry;
     NautilusLocationEntryPrivate *priv;
     GtkEditable *editable;
-    char *suffix, *user_location, *absolute_location, *uri_scheme;
-    int user_location_length, pos;
+    g_autofree char *absolute_location = NULL;
+    g_autofree char *user_location = NULL;
+    gboolean is_relative = FALSE;
+    int start_sel;
+    g_autofree char *uri_scheme = NULL;
+    g_auto (GStrv) completions = NULL;
+    char *completion;
+    int i;
+    GtkTreeIter iter;
+    int current_dir_strlen;
 
     entry = NAUTILUS_LOCATION_ENTRY (callback_data);
     priv = nautilus_location_entry_get_instance_private (entry);
     editable = GTK_EDITABLE (entry);
-    user_location = gtk_editable_get_chars (editable, 0, -1);
-    user_location_length = g_utf8_strlen (user_location, -1);
-    user_location = g_strchug (user_location);
-    user_location = g_strchomp (user_location);
+
+    if (gtk_editable_get_selection_bounds (editable, &start_sel, NULL))
+    {
+        user_location = gtk_editable_get_chars (editable, 0, start_sel);
+    }
+    else
+    {
+        user_location = gtk_editable_get_chars (editable, 0, -1);
+    }
+
+    g_strstrip (user_location);
+    set_prefix_dimming (priv->completion_cell, user_location);
+
     priv->idle_id = 0;
 
     uri_scheme = g_uri_parse_scheme (user_location);
 
     if (!g_path_is_absolute (user_location) && uri_scheme == NULL && user_location[0] != '~')
     {
+        is_relative = TRUE;
         absolute_location = g_build_filename (priv->current_directory, user_location, NULL);
-        suffix = g_filename_completer_get_completion_suffix (priv->completer,
-                                                             absolute_location);
-        g_free (absolute_location);
     }
     else
     {
-        suffix = g_filename_completer_get_completion_suffix (priv->completer,
-                                                             user_location);
+        absolute_location = g_steal_pointer (&user_location);
     }
 
-    g_free (user_location);
-    g_free (uri_scheme);
+    completions = g_filename_completer_get_completions (priv->completer, absolute_location);
 
-    /* if we've got something, add it to the entry */
-    if (suffix != NULL)
+    /* populate the completions model */
+    gtk_list_store_clear (priv->completions_store);
+
+    current_dir_strlen = strlen (priv->current_directory);
+    for (i = 0; completions[i] != NULL; i++)
     {
-        pos = user_location_length;
-        gtk_editable_insert_text (editable,
-                                  suffix, -1, &pos);
-        pos = user_location_length;
-        gtk_editable_select_region (editable, pos, -1);
+        completion = completions[i];
 
-        g_free (suffix);
+        if (is_relative && strlen (completion) >= current_dir_strlen)
+        {
+            /* For relative paths, we need to strip the current directory
+             * (and the trailing slash) so the completions will match what's
+             * in the text entry */
+            completion += current_dir_strlen;
+            if (G_IS_DIR_SEPARATOR (completion[0]))
+            {
+                completion++;
+            }
+        }
+
+        gtk_list_store_append (priv->completions_store, &iter);
+        gtk_list_store_set (priv->completions_store, &iter, 0, completion, -1);
+    }
+
+    /* refilter the completions dropdown */
+    gtk_entry_completion_complete (priv->completion);
+
+    if (priv->idle_insert_completion)
+    {
+        /* insert the completion */
+        gtk_entry_completion_insert_prefix (priv->completion);
     }
 
     return FALSE;
@@ -450,6 +515,7 @@ entry_would_have_inserted_characters (const GdkEvent *event)
         }
 
         default:
+        {
             if (keyval >= 0x20 && keyval <= 0xFF)
             {
                 if ((state & GDK_CONTROL_MASK) != 0)
@@ -461,6 +527,7 @@ entry_would_have_inserted_characters (const GdkEvent *event)
                     return FALSE;
                 }
             }
+        }
     }
 
     /* GTK+ 4 TODO: gdk_event_get_string () and check if length > 0. */
@@ -497,7 +564,7 @@ got_completion_data_callback (GFilenameCompleter    *completer,
         g_source_remove (priv->idle_id);
         priv->idle_id = 0;
     }
-    try_to_expand_path (entry);
+    update_completions_store (entry);
 }
 
 static void
@@ -513,6 +580,8 @@ finalize (GObject *object)
     g_free (priv->special_text);
 
     g_clear_object (&priv->last_location);
+    g_clear_object (&priv->completion);
+    g_clear_object (&priv->completions_store);
 
     G_OBJECT_CLASS (nautilus_location_entry_parent_class)->finalize (object);
 }
@@ -606,7 +675,9 @@ nautilus_location_entry_icon_release (GtkEntry             *gentry,
         break;
 
         default:
+        {
             g_assert_not_reached ();
+        }
     }
 }
 
@@ -655,12 +726,19 @@ nautilus_location_entry_on_event (GtkWidget *widget,
      * should position the insertion point at the end of
      * the selection.
      */
-    if (keyval == GDK_KEY_Tab && selected)
+    if (keyval == GDK_KEY_Tab && !(state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
     {
-        int position;
+        if (selected)
+        {
+            int position;
 
-        position = strlen (gtk_entry_get_text (GTK_ENTRY (editable)));
-        gtk_editable_select_region (editable, position, position);
+            position = strlen (gtk_entry_get_text (GTK_ENTRY (editable)));
+            gtk_editable_select_region (editable, position, position);
+        }
+        else
+        {
+            gtk_widget_error_bell (GTK_WIDGET (entry));
+        }
 
         return GDK_EVENT_STOP;
     }
@@ -679,19 +757,30 @@ nautilus_location_entry_on_event (GtkWidget *widget,
      */
     handled = parent_widget_class->key_press_event (widget, (GdkEventKey *) event);
 
-    /* Only do expanding when we are typing at the end of the
+
+    if (keyval == GDK_KEY_Down || keyval == GDK_KEY_Up)
+    {
+        /* Ignore up/down arrow keys. These are used by the entry completion,
+         * and if we modify the completion store, navigation through the list
+         * will be interrupted. */
+        return GDK_EVENT_PROPAGATE;
+    }
+
+    /* Only do completions when we are typing at the end of the
      * text. Do the expand at idle time to avoid slowing down
-     * typing when the directory is large. Only trigger the expand
+     * typing when the directory is large. Only insert an expansion
      * when we type a key that would have inserted characters.
      */
     if (position_and_selection_are_at_end (editable))
     {
-        if (entry_would_have_inserted_characters (event))
+        /* Only insert a completion if a character was typed. Otherwise,
+         * update the completions store (i.e. in case backspace was pressed)
+         * but don't insert the completion into the entry. */
+        priv->idle_insert_completion = entry_would_have_inserted_characters (event);
+
+        if (priv->idle_id == 0)
         {
-            if (priv->idle_id == 0)
-            {
-                priv->idle_id = g_idle_add (try_to_expand_path, widget);
-            }
+            priv->idle_id = g_idle_add (update_completions_store, widget);
         }
     }
     else
@@ -827,7 +916,9 @@ nautilus_location_entry_set_secondary_action (NautilusLocationEntry       *entry
         break;
 
         default:
+        {
             g_assert_not_reached ();
+        }
     }
     priv->secondary_action = secondary_action;
 }
@@ -863,19 +954,12 @@ static void
 nautilus_location_entry_init (NautilusLocationEntry *entry)
 {
     NautilusLocationEntryPrivate *priv;
-    GtkTargetList *targetlist;
 
     priv = nautilus_location_entry_get_instance_private (entry);
 
     priv->completer = g_filename_completer_new ();
     g_filename_completer_set_dirs_only (priv->completer, TRUE);
 
-    gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY, "folder-symbolic");
-    gtk_entry_set_icon_activatable (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY, FALSE);
-    targetlist = gtk_target_list_new (drag_types, G_N_ELEMENTS (drag_types));
-    gtk_entry_set_icon_drag_source (GTK_ENTRY (entry), GTK_ENTRY_ICON_PRIMARY, targetlist, GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
-    gtk_target_list_unref (targetlist);
-
     nautilus_location_entry_set_secondary_action (entry,
                                                   NAUTILUS_LOCATION_ENTRY_ACTION_CLEAR);
 
@@ -907,6 +991,25 @@ nautilus_location_entry_init (NautilusLocationEntry *entry)
                              G_CALLBACK (editable_activate_callback), entry, G_CONNECT_AFTER);
     g_signal_connect_object (entry, "changed",
                              G_CALLBACK (editable_changed_callback), entry, 0);
+
+    priv->completion = gtk_entry_completion_new ();
+    priv->completions_store = gtk_list_store_new (1, G_TYPE_STRING);
+    gtk_entry_completion_set_model (priv->completion, GTK_TREE_MODEL (priv->completions_store));
+
+    g_object_set (priv->completion,
+                  "text-column", 0,
+                  "inline-completion", TRUE,
+                  "inline-selection", TRUE,
+                  "popup-single-match", TRUE,
+                  NULL);
+
+    priv->completion_cell = gtk_cell_renderer_text_new ();
+    g_object_set (priv->completion_cell, "xpad", 6, NULL);
+
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->completion), priv->completion_cell, FALSE);
+    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->completion), priv->completion_cell, "text", 0);
+
+    gtk_entry_set_completion (GTK_ENTRY (entry), priv->completion);
 }
 
 GtkWidget *
