@@ -225,11 +225,13 @@ vfs_file_get_date (NautilusFile     *file,
 {
     time_t atime;
     time_t mtime;
+    time_t btime;
     time_t recency;
     time_t trash_time;
 
     atime = nautilus_file_get_atime (file);
     mtime = nautilus_file_get_mtime (file);
+    btime = nautilus_file_get_btime (file);
     recency = nautilus_file_get_recency (file);
     trash_time = nautilus_file_get_trash_time (file);
 
@@ -263,6 +265,20 @@ vfs_file_get_date (NautilusFile     *file,
             return TRUE;
         }
 
+        case NAUTILUS_DATE_TYPE_CREATED:
+        {
+            /* Before we have info on a file, the date is unknown. */
+            if (btime == 0)
+            {
+                return FALSE;
+            }
+            if (date != NULL)
+            {
+                *date = btime;
+            }
+            return TRUE;
+        }
+
         case NAUTILUS_DATE_TYPE_TRASHED:
         {
             /* Before we have info on a file, the date is unknown. */
@@ -278,6 +294,7 @@ vfs_file_get_date (NautilusFile     *file,
         }
 
         case NAUTILUS_DATE_TYPE_RECENCY:
+        {
             /* Before we have info on a file, the date is unknown. */
             if (recency == 0)
             {
@@ -288,6 +305,7 @@ vfs_file_get_date (NautilusFile     *file,
                 *date = recency;
             }
             return TRUE;
+        }
     }
     return FALSE;
 }
