@@ -35,7 +35,7 @@ typedef enum {
 } NautilusLocationChangeType;
 
 #define NAUTILUS_TYPE_WINDOW_SLOT (nautilus_window_slot_get_type ())
-G_DECLARE_DERIVABLE_TYPE (NautilusWindowSlot, nautilus_window_slot, NAUTILUS, WINDOW_SLOT, GtkBox)
+G_DECLARE_FINAL_TYPE (NautilusWindowSlot, nautilus_window_slot, NAUTILUS, WINDOW_SLOT, GtkBox)
 
 typedef struct
 {
@@ -46,26 +46,6 @@ typedef struct
     NautilusBookmark *current_location_bookmark;
 } NautilusNavigationState;
 
-struct _NautilusWindowSlotClass {
-	GtkBoxClass parent_class;
-
-	/* wrapped NautilusWindowInfo signals, for overloading */
-	void (* active)   (NautilusWindowSlot *slot);
-	void (* inactive) (NautilusWindowSlot *slot);
-
-        /* Use this in case the subclassed slot has some special views differents
-         * that the ones supported here. You can return your nautilus view
-         * subclass in this function.
-         */
-        NautilusView*  (* get_view_for_location) (NautilusWindowSlot *slot,
-                                                  GFile              *location);
-        /* Whether this type of slot handles the location or not. This can be used
-         * for the special slots which handle special locations like the desktop
-         * or the other locations. */
-        gboolean (* handles_location) (NautilusWindowSlot *slot,
-                                       GFile              *location);
-};
-
 NautilusWindowSlot * nautilus_window_slot_new              (NautilusWindow     *window);
 
 NautilusWindow * nautilus_window_slot_get_window           (NautilusWindowSlot *slot);
@@ -77,14 +57,6 @@ void nautilus_window_slot_open_location_full              (NautilusWindowSlot
                                                            NautilusWindowOpenFlags  flags,
                                                            GList                   *new_selection);
 
-void nautilus_window_slot_open_location_set_navigation_state (NautilusWindowSlot         *slot,
-                                                              GFile                      *location,
-                                                              NautilusWindowOpenFlags     flags,
-                                                              GList                      *new_selection,
-                                                              NautilusLocationChangeType  change_type,
-                                                              NautilusNavigationState    *navigation_state,
-                                                              guint                       distance);
-
 GFile * nautilus_window_slot_get_location		   (NautilusWindowSlot *slot);
 GFile * nautilus_window_slot_get_pending_location          (NautilusWindowSlot *slot);
 
@@ -129,9 +101,6 @@ GList* nautilus_window_slot_get_selection                  (NautilusWindowSlot *
 void     nautilus_window_slot_search                       (NautilusWindowSlot *slot,
                                                             NautilusQuery      *query);
 
-gboolean nautilus_window_slot_handles_location (NautilusWindowSlot *self,
-                                                GFile              *location);
-
 void nautilus_window_slot_restore_navigation_state (NautilusWindowSlot      *self,
                                                     NautilusNavigationState *data);
 
@@ -142,9 +111,8 @@ NautilusQueryEditor *nautilus_window_slot_get_query_editor (NautilusWindowSlot *
 /* Only used by slot-dnd */
 NautilusView*  nautilus_window_slot_get_current_view       (NautilusWindowSlot *slot);
 
-void nautilus_window_slot_back_or_forward                  (NautilusWindowSlot     *slot,
-                                                            gboolean                back,
-                                                            guint                   distance,
-                                                            NautilusWindowOpenFlags flags);
+void nautilus_window_slot_back_or_forward                  (NautilusWindowSlot *slot,
+                                                            gboolean            back,
+                                                            guint               distance);
 
 void free_navigation_state                                 (gpointer data);
